@@ -1,18 +1,9 @@
-use std::{
-	convert::TryFrom,
-	fmt,
-	sync::mpsc::Receiver,
-	thread,
-	time::Duration,
-};
+use std::{convert::TryFrom, fmt, sync::mpsc::Receiver, thread, time::Duration};
 
 use log::info;
 use midly::Timing;
 
-use crate::{
-	Event,
-	Moment,
-};
+use crate::{Event, Moment};
 
 /// Used for timing MIDI playback.
 pub trait Timer {
@@ -47,7 +38,7 @@ pub trait Timer {
 		if !t.is_zero() {
 			#[cfg(feature = "verbose-log")]
 			log::debug!(target: "Timer", "sleeping the thread for {:?}", &t);
-			thread::sleep(t);
+			sleep(t);
 		} else {
 			#[cfg(feature = "verbose-log")]
 			log::trace!(target: "Timer", "timer returned 0 duration, not sleeping")
@@ -132,6 +123,16 @@ impl Ticker {
 		s.change_tempo(tempo);
 		s
 	}
+
+	/// Upgrades `self` to a [ControlTicker].
+	pub fn to_control(self, pause: Receiver<()>) -> ControlTicker {
+		ControlTicker {
+			speed: self.speed,
+			micros_per_tick: self.micros_per_tick,
+			ticks_per_beat: self.ticks_per_beat,
+			pause,
+		}
+	}
 }
 
 impl Timer for Ticker {
@@ -251,6 +252,15 @@ impl ControlTicker {
 		s.change_tempo(tempo);
 		s
 	}
+
+	/// Casts `self` to a [Ticker].
+	pub fn as_ticker(&self) -> Ticker {
+		Ticker {
+			ticks_per_beat: self.ticks_per_beat,
+			micros_per_tick: self.micros_per_tick,
+			speed: self.speed,
+		}
+	}
 }
 
 impl Timer for ControlTicker {
@@ -298,10 +308,54 @@ impl Timer for ControlTicker {
 		if !t.is_zero() {
 			#[cfg(feature = "verbose-log")]
 			log::debug!(target: "Timer", "sleeping the thread for {:?}", &t);
-			thread::sleep(t);
+			sleep(t);
 		} else {
 			#[cfg(feature = "verbose-log")]
 			log::trace!(target: "Timer", "timer returned 0 duration, not sleeping")
 		}
 	}
 }
+
+/// Sleeps the thread with the given duration.
+///
+/// Sleeps with [thread::sleep] for the most of the time
+/// and spin-locks for the last T milliseconds, where T:
+/// - Windows: 15.
+/// - Non-Windows: 3.
+#[cfg(any(doc, test, feature = "hybrid-sleep"))]
+pub fn sleep(t: Duration) {
+	use std::time::Instant;
+	#[cfg(windows)]
+	const LIMIT: Duration = Duration::from_millis(15);
+	#[cfg(not(windows))]
+	const LIMIT: Duration = Duration::from_millis(3);
+
+	let t = if t < LIMIT {
+		t
+	} else {
+		let mut last = Instant::now();
+		let mut remaining = t;
+		loop {
+			thread::sleep(Duration::from_millis(1));
+			let now = Instant::now();
+			remaining = remaining.checked_sub(now - last).unwrap_or_default();
+			if remaining <= LIMIT {
+				break remaining;
+			}
+			last = now;
+		}
+	};
+	spin_lock(t);
+}
+
+#[cfg(feature = "hybrid-sleep")]
+#[inline]
+fn spin_lock(t: Duration) {
+	let now = std::time::Instant::now();
+	while now.elapsed() < t {}
+}
+
+#[cfg(not(any(doc, test, feature = "hybrid-sleep")))]
+fn sleep(t: Duration) {
+	thread::sleep(t);
+}
